
#include "render_area.hpp"

#include <GL/glu.h>
#include <iostream>
#include <cmath>

#include <QPainter>
#include <QMouseEvent>


render_area::render_area(QWidget *parent)
    :QWidget(parent)
{
    setBackgroundRole(QPalette::Base);
    setAutoFillBackground(true);

    dt=1/30.0f;
    connect(&timer, SIGNAL(timeout()), this, SLOT(update_timer()));
    timer.start(30);


    draw_circle=true;

    x_old=0;
    y_old=0;
    x_old2=0;
    y_old2=0;

    xc=200;
    yc=150;
    diametre=50;

    vxc=0.0f;
    vyc=0.0f;

    is_fixed=false;
}

render_area::~render_area()
{}



void render_area::paintEvent(QPaintEvent*)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing, true);

    QPen pen;
    pen.setWidth(1.0);
    pen.setColor(Qt::blue);
    painter.setPen(pen);

    QBrush brush = painter.brush();
    brush.setColor(Qt::gray);
    brush.setStyle(Qt::SolidPattern);
    painter.setBrush(brush);

    if(draw_circle)
        painter.drawEllipse(xc,yc,diametre,diametre);


}

void render_area::change_draw_circle()
{
    draw_circle=!draw_circle;
    repaint();
}

void render_area::mousePressEvent(QMouseEvent *event)
{
    x_old=event->x();
    y_old=event->y();

    is_fixed=true;
}

void render_area::mouseReleaseEvent(QMouseEvent *event)
{

    float x=event->x();
    float y=event->y();


    vxc=60*(x-x_old2);
    vyc=60*(y-y_old2);

    is_fixed=false;

}

void render_area::mouseMoveEvent(QMouseEvent *event)
{
    int x=event->x();
    int y=event->y();

    float R=diametre/2.0f;
    float x0=xc+R;
    float y0=yc+R;

    if( (x-x0)*(x-x0)+(y-y0)*(y-y0)<R*R )
    {
        xc+=(x-x_old);
        yc+=(y-y_old);
    }

    x_old2=x_old;
    y_old2=y_old;
    x_old=x;
    y_old=y;

    repaint();
}

void render_area::update_timer()
{

    float A=180.0f;

    if(is_fixed==false)
    {
        //gestion du sol
        if(yc+diametre>=height())
        {
            if(std::fabs(vyc)>A)
                vyc=-0.4*vyc;
            else
                vyc=0.0f;

            vxc*=0.9;
            yc=height()-diametre;
        }

        //integration (Euler explicite)
        xc=xc+dt*vxc;
        yc=yc+dt*vyc;

        vyc=vyc+dt*9.81*A;
    }


    repaint();
}


